@@ -105,14 +105,6 @@ rather than handled directly.
 
 =head1 ATTRIBUTES
 
-=head2 cluster (internally provided)
-
-The type of cluster that will be used to execute the group
-of stages.  This value is passed on by the $group->stage
-method when it creates a new stage.  Since it also uses that
-value to select the type of stage object that is created,
-it is somewhat redundant.
-
 =cut
 
 sub _build_valid_resources {
@@ -219,25 +211,6 @@ class_has '_default_retry_resources' => (
     builder => '_build_default_retry_resources',
 );
 
-has 'cluster' => (
-    is      => 'ro',
-    isa     => 'Str',
-);
-
-=head2 name
-
-The name of this stage within the group.  The name must be unique
-within the group, most group interactions use the name rather than
-a reference to the stage object to refer to the stage.
-
-=cut
-
-has 'name' => (
-    is       => 'ro',
-    isa      => 'Str',
-    required => 1,
-);
-
 =head2 command
 
 The command to be executed on the cluster.  This command will quite
@@ -594,336 +567,6 @@ sub default_args_string_parsing_info {
 }
 
 
-=head2 group
-
-The group that this stage belongs to is automatically provided to
-the stage creation.  You don't need to initialize it, and since
-you usually will not be working with Stage objects directly, you
-won't have much need to use it either.
-
-=cut
-
-has 'group' => (
-    is       => 'ro',
-    isa      => 'Object',
-    weak_ref => 1,
-    required => 1,
-    handles  => {
-        debug       => 'debug',
-        info        => 'info',
-        warn        => 'warn',
-        error       => 'error',
-        fatal       => 'fatal',
-
-        file_params => 'file_params',
-        _file_info  => '_file_info',
-    },
-);
-
-sub _croak {
-    my $self = shift;
-    my $msg  = shift;
-    $msg = "Stage(" . $self->name . "): $msg";
-    $self->fatal($msg);
-    confess $msg;
-}
-
-=head2 storage_class (internal)
-
-The name of key to use to select a storage class from the storage_classes
-attribute for files that do no have an explicit class given.
-
-Defaults to the value of the group's C<storage_class> attribute, (which
-defaults to C<'default'>) - this value is provided automatically by the
-group's stage method if no explicit storage_class value is provided for
-the stage.
-
-=cut
-
-has 'storage_class' => (
-    is       => 'ro',
-    isa      => 'HPCIFileGen',
-	lazy     => 1,
-	default  => sub {
-        $_[0]->group->storage_class
-    },
-    required => 1,
-);
-
-# has 'storage_classes' => (
-# 	is       => 'ro',
-# 	isa      => 'HPCIListList',
-# 	lazy     => 1,
-# 	default  => sub { $_[0]->group->storage_classes },
-# 	init_arg => undef,
-# );
-
-=head2 files
-
-A hash that can contain lists of files.
-
-Throughout this hash, there are filenames contained within hash elements
-that describe the processing required for that file.  Whenever a filename
-is needed, it can either be a string containing a pathname, or it can be
-an HPCI::File object (or subclass), or ot can be a HashRef.  Often, it
-will be the string form, which will be converted to an object internally.
-
-The top level of the hash has keys 'in' (for input), 'out' (for output),
-'skipstage', 'rename', and 'delete'.
-(The same file might be listed under multiple keys.)
-
-The values for these keys are:
-
-=over 4
-
-=item 'in'
-
-a hashref with possible keys:
-
-    'req' (for required input files)
-    'opt' (for required output files)
-
-The value for either of these can be either a filename or a list of filenames.
-
-=item 'out'
-
-a hashref with possible keys:
-
-    'req' (for required output files)
-    'opt' (for required output files)
-
-The value for either of these can be either a filename or a list of filenames.
-
-=item 'skipstage'
-
-either:
-
-=over 4
-
-=item
-
-an arrayref
-
-=item
-
-a hashref with the keys 'pre' (for pre-requisites) and 'lists'
-
-=back
-
-The arrayref (either the arrayref value of 'skipstage' or the arrayref value
-for the 'lists' hash element) can contain either a list of files, or a hashref
-with keys 'pre' and 'files'.
-
-The 'pre' value (if present) at the top level is a list of files which are
-pre-requisites for all of the lists.  If a list has its own 'pre' list, those
-files are only pre-requities for the files in that list.
-
-=item 'rename'
-
-    a list of pairs of filenames
-
-The file named as the first element in each pair (if it exists) is renamed to
-the second filename in the pair.  It is not considered an error for the first
-file in a pair does not exist - if you want to ensure that a file exists,
-include it as an 'out'->'req' file as well.
-
-=item 'delete'
-
-can be either:
-
-    a scalar filename
-    a list of filenames
-
-These will be removed if the stage completes successfully.  It is not
-considered an error if any of these files does not exist - include them
-in the 'out'->'req' files list if you wish to ensure that they do.
-
-=back
-
-The contents are used at various times:
-
-=over 4
-
-=item the stage is ready to be executed
-
-=over
-
-=item
-
-if a 'skipstage' key is present then checking is done to decide whether the
-stage needs to be executed or can be skipped (treating it as a successful
-completion)
-
-the main content of this key is a list of lists of filenames (the target
-files) - if any of these lists has all of its files existing, then the stage
-can be skipped
-
-if there is a top level and/or a list level 'pre' list, then all of the files
-in the pre list(s) must also exist and be older than the target files (the
-files in the top level 'pre' list are checked against all of the target lists,
-the files in a target level 'pre' list are only checked against that target).
-
-C<skipstage> checking is always done by the parent process, in hopes of
-avoiding the need to create the stage.
-
-=item
-
-all 'in'->'req' files must exist, if any is missing, the stage is aborted.
-If the files exist, then the child stage will be set up (if needed) to
-download those files from the long-term storage.
-
-=item
-
-all 'in'->'opt' are checked by the parent. If any exists, then the child stage
-will be set up (if needed) to download them from the long-term storage.
-
-=back
-
-=item the stage has completed execution
-
-=over
-
-=item
-
-all 'out'->'req' files must exists and they must have been updated during the
-execution of the stage (otherwise the stage is treated as failing)
-
-=item
-
-any 'out'->'opt' files which exist must have been updated during the execution
-of the stage (otherwise the stage is treated as failing)
-
-=item
-
-clusters that require special treatment of files can take copying actions to
-collect any 'out' files that have been updated
-
-=item
-
-if the stage completed successfully, any files lists as 'delete' are removed
-
-=back
-
-=back
-
-=cut
-
-has 'files' => (
-    is        => 'ro',
-    isa       => 'Maybe[HashRef]',
-    default   => undef,
-);
-
-has '_use_files' => (
-    is        => 'ro',
-    isa       => 'Maybe[HashRef]',
-    lazy      => 1,
-    init_arg  => undef,
-    builder   => '_use_files_builder',
-);
-
-sub _use_files_builder {
-    my $self      = shift;
-    my $file      = $self->files // return undef;
-    my $use_files = {};
-    SIMPLE_FILES:
-    for my $keys (  [qw(in req)],
-                    [qw(in opt)],
-                    [qw(out req)],
-                    [qw(out opt)],
-                    ['delete']
-                ) {
-        my $source = $file;
-        $source = $source->{$_} or next SIMPLE_FILES for @$keys;
-        my $prev_dest;
-        my $dest = $use_files;
-        for my $key (@$keys) {
-            $prev_dest = $dest;
-            $dest = $dest->{$key} //= {};
-        }
-        $prev_dest->{$keys->[-1]} = $self->_get_file_list($source);
-    }
-    if ( my $skipinfo = $file->{skipstage} ) {
-        $use_files->{skipstage} = {};
-        if (ref($skipinfo) eq 'HASH') {
-            $self->_croak('skipstage hash element must be have keys "pre" and "lists"')
-                unless exists $skipinfo->{pre} && exists $skipinfo->{lists};
-            $use_files->{skipstage}{pre} = $self->_get_file_list( $skipinfo->{pre} );
-            $skipinfo = $skipinfo->{lists};
-            $self->_croak('skipstage->{lists} element must be an array')
-                unless ref($skipinfo) eq 'ARRAY';
-        }
-        else {
-            $self->_croak('skipstage files element must be either a hash or an array')
-                unless ref($skipinfo) eq 'ARRAY';
-            $use_files->{skipstage}{pre} = [];
-        }
-        my $dest_list = $use_files->{skipstage}{lists} = [ ];
-        for my $l (@$skipinfo) {
-            my $dest_hash = { };
-            $self->_croak('skipstage list element must be either a hash or an array')
-                unless ref($l) eq 'ARRAY' || ref($l) eq 'HASH';
-            if (ref($l) eq 'HASH') {
-                $dest_hash->{pre} = $self->_get_file_list( $l->{pre} );
-                $l = $l->{post};
-                $self->_croak('skipstage post element must be an array')
-                    unless ref($l) eq 'ARRAY';
-            }
-            else {
-                $dest_hash->{pre} = [];
-            }
-            $dest_hash->{post} = $self->_get_file_list( $l );
-            push @$dest_list, $dest_hash;
-        }
-    }
-    if ( my $renamelist = $file->{rename} ) {
-        $self->_croak('rename files element an array')
-            unless ref($renamelist) eq 'ARRAY';
-        my $use_pairs = $use_files->{rename} = [];
-        for my $pair (@$renamelist) {
-            $self->_croak('rename files array element must be an array with two values')
-                unless ref($pair) eq 'ARRAY' && scalar(@$pair) == 2;
-            my ($from, $to) = @$pair;
-            push @$use_pairs, [ $self->_get_file_obj($from), $to ];
-        }
-    }
-    return $use_files;
-};
-
-sub _get_file_list {
-    my $self   = shift;
-    my $source = shift;
-    $source = [ $source ] unless ref($source);
-    $source = [ $source ] if blessed $source && $source->isa("HPCI::File");
-    my $ref = ref($source);
-    $self->_croak( "Element in files attribute is not an array, a pathname or HPCI::File class. Type is $ref, value is $source")
-        unless $ref eq 'ARRAY';
-    return [ map { $self->_get_file_obj($_) } @$source ];
-}
-
-sub _get_file_obj {
-    my $self = shift;
-    my $file = shift;
-    return $self->_generate_file( ref($file) eq 'ARRAY' ? @$file : $file );
-}
-
-sub _generate_file {
-    my $self = shift;
-    my $file = shift;
-    if (blessed $file && $file->isa("HPCI::File")) {
-        return $file;
-    }
-    my $path = File::Spec->rel2abs("$file");
-    # TODO: add search through storage_classes list here
-    my $known = $self->_file_info->{$path} //= {};
-    return $known->{file} //= $self->storage_class->(
-        $path,
-        stage => $self,
-        %{ $known->{params} // {} },
-        @_
-    );
-}
-
 =head2 state, is_ready, is_blocked, is_pass, is_fail
 
 These states are mostly for internal use during execution.  (There
@@ -1133,122 +776,6 @@ after '_analyse_completion_state' => sub {
     }
 };
 
-sub lof {
-    my $lof = shift;
-    my $res = '[';
-    $res .= " $_" for @$lof;
-    $res .= ' ]';
-    return $res;
-}
-
-# check whether criteria are satisfied to skip executing the stage
-sub _can_be_skipped {
-    my $self = shift;
-    my $skipinfo = $self->_use_files   // return 0;
-    $skipinfo    = $skipinfo->{skipstage} // return 0;
-    my $pre      = $skipinfo->{pre};;
-    my $lol      = $skipinfo->{lists};
-  LIST:
-    for my $l (@$lol) {
-        my $thispre = [@$pre, @{$l->{pre}}];
-        $l = $l->{post};
-        $self->debug( "Considering skipstage, pre: ", lof($thispre), ", list: ", lof( $l ) );
-        unless (@$l) { # an empty list does not qualify for skipping
-            $self->debug( "no skip: empty list element" );
-            next LIST;
-        }
-        my $newest_pre;
-        for my $f (@$thispre) {
-            # if there is a pre list
-            #     check that they all exist and
-            #     get latest pre timestamp
-            my $ts = $f->timestamp;
-            unless (defined $ts) {
-                $self->debug( "no skip: missing pre element: $f" );
-                next LIST;
-            }
-            $newest_pre ||= $ts;
-            $newest_pre = $ts if $ts < $newest_pre;
-        }
-        if ($self->_file_list_acceptable_for_skip( $newest_pre, $l )) {
-            $self->debug( "skipping: file list acceptable" );
-            return 1;
-        }
-        $self->debug( "no skip: file list not acceptable" );
-    }
-    return 0;
-}
-
-sub _files_ready_to_start_stage {
-    my $self = shift;
-    return 1 unless my $files = $self->_use_files;
-    my $retval = 1; # success unless missing file(s) found
-    if (my $in = $files->{in}) {
-        while ( my ($type, $fileval) = each %$in ) {
-            for my $file (@$fileval) {
-                unless ($file->valid_in_file || $type ne 'req') {
-                # need to check exist on opt files in case
-                #   the driver needs to take special action to
-                #   make it available to the stage when it runs
-                #
-                # go through the entire list of files to give user
-                # a full list of missing files rather than just the
-                # first one we notice - let them fix everything for
-                # the next run instead of needing separate extra
-                # funs to be notified of each successive issue
-                    $self->error("Required input file ($file) not present");
-                    $self->_set_failure_info(
-                        "Failed stage ("
-                        . $self->name
-                        . ") without execution because one or more required input files were not present"
-                    );
-                    $retval = 0;
-                }
-            }
-        }
-    }
-    if ($retval and my $unshared = $files->{unshared}) {
-        # our @HPCI::ScriptSource::pre_commands;
-        # our @HPCI::ScriptSource::post_success_commands;
-        if (my $in = $unshared->{in}) {
-            while ( my ($type, $fileval) = each %$in ) {
-                for my $file (@$fileval) {
-                    my $exists = $self->_unshared_file_exists_for_in($file);
-                    my $get = $self->_unshared_file_download_for_in($file);
-                    push @HPCI::ScriptSource::pre_commands, "if $exists\n";
-                    push @HPCI::ScriptSource::pre_commands, "then\n";
-                    push @HPCI::ScriptSource::pre_commands, "    $get\n";
-                    if ($type eq 'req') {
-                        push @HPCI::ScriptSource::pre_commands, "else\n";
-                        push @HPCI::ScriptSource::pre_commands, "    echo required in file not present: $file 1>&2\n";
-                        push @HPCI::ScriptSource::pre_commands, "    exit 126\n";
-                    }
-                    push @HPCI::ScriptSource::pre_commands, "fi\n";
-                }
-            }
-        }
-        if (my $out = $unshared->{out}) {
-            while ( my ($type, $fileval) = each %$out ) {
-                # for my $file ($self->_scalar_list($fileval)) {
-                for my $file (@$fileval) {
-                    my $exists = $self->_unshared_file_exists_for_out($file);
-                    my $put = $self->_unshared_file_upload_for_out($file);
-                    push @HPCI::ScriptSource::post_success_commands, "if $exists\n";
-                    push @HPCI::ScriptSource::post_success_commands, "then\n";
-                    push @HPCI::ScriptSource::post_success_commands, "    $put\n";
-                    if ($type eq 'req') {
-                        push @HPCI::ScriptSource::post_success_commands, "else\n";
-                        push @HPCI::ScriptSource::post_success_commands, "    echo required out file not present: $file 1>&2\n";
-                        push @HPCI::ScriptSource::post_success_commands, "    exit 126\n";
-                    }
-                    push @HPCI::ScriptSource::post_success_commands, "fi\n";
-                }
-            }
-        }
-    }
-    return $retval;
-}
-
 # sub _files_acceptable_for_completed_stage {
 #     my $self = shift;
 #     my $run  = shift;
@@ -1512,7 +1039,7 @@ sub get_run_stats {
     my $self = shift;
     return [
         $self->_has_failure_info          ? { exit_status => $self->_failure_info }
-        : scalar( @{ $self->runs } ) == 0 ? { exit_status => 'Stage skipped, skipstage determined it was complete',
+        : scalar( @{ $self->runs } ) == 0 ? { exit_status => 'Stage skipped, skip determined it was complete',
                                               final_job_state => 'pass'
                                             }
         :                                   map { $_->stats } @{ $self->runs }
@@ -1538,7 +1065,7 @@ which are not dependent upon this stage.
 
 If the stage fails, then no other stages are started.  The group
 simply waits until previously started stages complete and then
-returns.
+completes execution.
 
 =item - ignore
 
@@ -1557,7 +1084,7 @@ has 'failure_action' => (
     default  => 'abort_deps',
 );
 
-with qw(HPCI::Env HPCI::CommandScript HPCI::ScriptSource);
+with qw(HPCI::Sub HPCI::SuperSub HPCI::Env HPCI::CommandScript HPCI::ScriptSource);
 
 =head1 METHODS
 
